@@ -319,7 +319,7 @@ static void set_path(GdiCanvas *canvas, SpicePath *s)
     unsigned int i;
 
     for (i = 0; i < s->num_segments; i++) {
-        SpicePathSeg* seg = s->segments[0];
+        SpicePathSeg* seg = s->segments[i];
         SpicePointFix* point = seg->points;
         SpicePointFix* end_point = point + seg->count;
 
@@ -564,7 +564,7 @@ static uint8_t *create_bitmap(HBITMAP *bitmap, HBITMAP *prev_bitmap, HDC *dc,
         break;
     default:
         spice_warn_if_reached();
-        return;
+        return NULL;
     }
 
     if (bitmap_data) {
@@ -756,7 +756,7 @@ uint8_t calc_rop3(uint16_t rop3_bits, int brush)
 
     if (rop3_bits & SPICE_ROPD_OP_BLACKNESS || rop3_bits & SPICE_ROPD_OP_WHITENESS ||
         rop3_bits & SPICE_ROPD_OP_INVERS) {
-        spice_warn_if_reached("invalid rop3 type");
+        spice_warning("invalid rop3 type");
         return 0;
     }
     return rop3;
@@ -1802,11 +1802,9 @@ static SpiceCanvasOps gdi_canvas_ops;
 
 SpiceCanvas *gdi_canvas_create(int width, int height,
                                HDC dc, RecurciveMutex* lock, uint32_t format
+                             , SpiceImageCache *bits_cache
 #ifdef SW_CANVAS_CACHE
-                             , SpiceImageCache *bits_cache
                              , SpicePaletteCache *palette_cache
-#elif defined(SW_CANVAS_IMAGE_CACHE)
-                             , SpiceImageCache *bits_cache
 #endif
                             , SpiceImageSurfaces *surfaces
                             , SpiceGlzDecoder *glz_decoder
@@ -1822,11 +1820,9 @@ SpiceCanvas *gdi_canvas_create(int width, int height,
     canvas = spice_new0(GdiCanvas, 1);
     canvas_base_init(&canvas->base, &gdi_canvas_ops,
                      width, height, format,
+                     bits_cache,
 #ifdef SW_CANVAS_CACHE
-                     bits_cache,
                      palette_cache,
-#elif defined(SW_CANVAS_IMAGE_CACHE)
-                     bits_cache,
 #endif
                      surfaces,
                      glz_decoder,
