@@ -117,6 +117,7 @@ struct _SpiceSessionPrivate {
     uint32_t          n_display_channels;
     guint8            uuid[16];
     gchar             *name;
+    SpiceImageCompression preferred_compression;
 
     /* associated objects */
     SpiceAudio        *audio_manager;
@@ -203,6 +204,7 @@ enum {
     PROP_SHARE_DIR_RO,
     PROP_USERNAME,
     PROP_UNIX_PATH,
+    PROP_PREF_COMPRESSION,
 };
 
 /* signals */
@@ -213,6 +215,38 @@ enum {
     SPICE_SESSION_LAST_SIGNAL,
 };
 
+/* Register SpiceImageCompress */
+#define SPICE_TYPE_IMAGE_COMPRESSION spice_image_compress_get_type()
+GType spice_image_compress_get_type (void);
+
+static const GEnumValue _spice_image_compress_values[] = {
+  { SPICE_IMAGE_COMPRESSION_INVALID, "SPICE_IMAGE_COMPRESSION_INVALID", "invalid" },
+  { SPICE_IMAGE_COMPRESSION_OFF, "SPICE_IMAGE_COMPRESSION_OFF", "off" },
+  { SPICE_IMAGE_COMPRESSION_AUTO_GLZ, "SPICE_IMAGE_COMPRESSION_AUTO_GLZ", "auto-glz" },
+  { SPICE_IMAGE_COMPRESSION_AUTO_LZ, "SPICE_IMAGE_COMPRESSION_AUTO_LZ", "auto-lz" },
+  { SPICE_IMAGE_COMPRESSION_QUIC, "SPICE_IMAGE_COMPRESSION_QUIC", "quic" },
+  { SPICE_IMAGE_COMPRESSION_GLZ, "SPICE_IMAGE_COMPRESSION_GLZ", "glz" },
+  { SPICE_IMAGE_COMPRESSION_LZ, "SPICE_IMAGE_COMPRESSION_LZ", "lz" },
+  { SPICE_IMAGE_COMPRESSION_LZ4, "SPICE_IMAGE_COMPRESSION_LZ4", "lz4" },
+  { 0, NULL, NULL }
+};
+
+G_STATIC_ASSERT(G_N_ELEMENTS(_spice_image_compress_values) == SPICE_IMAGE_COMPRESSION_ENUM_END + 1);
+
+GType
+spice_image_compress_get_type (void)
+{
+  static GType type = 0;
+  static volatile gsize type_volatile = 0;
+
+  if (g_once_init_enter(&type_volatile)) {
+    type = g_enum_register_static ("SpiceImageCompress", _spice_image_compress_values);
+    g_once_init_leave(&type_volatile, type);
+  }
+
+  return type;
+}
+
 static guint signals[SPICE_SESSION_LAST_SIGNAL];
 
 static void spice_session_channel_destroy(SpiceSession *session, SpiceChannel *channel);
@@ -257,7 +291,7 @@ static void spice_session_init(SpiceSession *session)
     g_free(channels);
 
     ring_init(&s->channels);
-    s->images = cache_new((GDestroyNotify)pixman_image_unref);
+    s->images = cache_image_new((GDestroyNotify)pixman_image_unref);
     s->glz_window = glz_decoder_window_new();
     update_proxy(session, NULL);
 }
@@ -658,6 +692,9 @@ static void spice_session_get_property(GObject    *gobject,
     case PROP_SHARE_DIR_RO:
         g_value_set_boolean(value, s->share_dir_ro);
         break;
+    case PROP_PREF_COMPRESSION:
+        g_value_set_enum(value, s->preferred_compression);
+        break;
     default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
 	break;
@@ -794,6 +831,9 @@ static void spice_session_set_property(GObject      *gobject,
     case PROP_SHARE_DIR_RO:
         s->share_dir_ro = g_value_get_boolean(value);
         break;
+    case PROP_PREF_COMPRESSION:
+        s->preferred_compression = g_value_get_enum(value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, pspec);
         break;
@@ -1403,6 +1443,24 @@ static void spice_session_class_init(SpiceSessionClass *klass)
                               G_PARAM_CONSTRUCT |
                               G_PARAM_STATIC_STRINGS));
 
+    /**
+     * SpiceSession:preferred-compression:
+     *
+     * The image compression algorithm the client prefers to use. It is
+     * reported to the server.
+     *
+     * Since: 0.29
+     **/
+    g_object_class_install_property
+        (gobject_class, PROP_PREF_COMPRESSION,
+         g_param_spec_enum("preferred-compression",
+                           "Preferred image compression algorithm",
+                           "Preferred image compression algorithm",
+                           SPICE_TYPE_IMAGE_COMPRESSION,
+                           SPICE_IMAGE_COMPRESSION_INVALID,
+                           G_PARAM_READWRITE |
+                           G_PARAM_STATIC_STRINGS));
+
     g_type_class_add_private(klass, sizeof(SpiceSessionPrivate));
 }
 
@@ -1484,12 +1542,16 @@ SpiceSession *spice_session_new_from_session(SpiceSession *session)
 
 /**
  * spice_session_connect:
- * @session:
+ * @session: a #SpiceSession
  *
  * Open the session using the #SpiceSession:host and
  * #SpiceSession:port.
  *
- * Returns: %FALSE if the connection failed.
+ * Returns: %FALSE if the session state is invalid for connection
+ * request. %TRUE if the connection is initiated. To know whether the
+ * connection is established, you must watch for channels creation
+ * (#SpiceSession::channel-new) and the channels state
+ * (#SpiceChannel::channel-event).
  **/
 gboolean spice_session_connect(SpiceSession *session)
 {
@@ -1513,7 +1575,7 @@ gboolean spice_session_connect(SpiceSession *session)
 
 /**
  * spice_session_open_fd:
- * @session:
+ * @session: a #SpiceSession
  * @fd: a file descriptor (socket) or -1
  *
  * Open the session using the provided @fd socket file
@@ -1531,7 +1593,7 @@ gboolean spice_session_connect(SpiceSession *session)
  * @fd this call since you will have to hook to SpiceChannel::open-fd signal
  * anyway.
  *
- * Returns:
+ * Returns: %TRUE on success.
  **/
 gboolean spice_session_open_fd(SpiceSession *session, int fd)
 {
@@ -1837,7 +1899,9 @@ void spice_session_migrate_end(SpiceSession *self)
  * spice_session_get_read_only:
  * @session: a #SpiceSession
  *
- * Returns: wether the @session is in read-only mode.
+ * Checks whether the @session is read-only.
+ *
+ * Returns: whether the @session is in read-only mode.
  **/
 gboolean spice_session_get_read_only(SpiceSession *self)
 {
@@ -1860,7 +1924,7 @@ static gboolean session_disconnect_idle(SpiceSession *self)
 
 /**
  * spice_session_disconnect:
- * @session:
+ * @session: a #SpiceSession
  *
  * Disconnect the @session, and destroy all channels.
  **/
@@ -1913,6 +1977,7 @@ GList *spice_session_get_channels(SpiceSession *session)
 /**
  * spice_session_has_channel_type:
  * @session: a #SpiceSession
+ * @type: a #SpiceChannel:channel-type
  *
  * See if there is a @type channel in the channels associated with this
  * @session.
@@ -2543,6 +2608,8 @@ void spice_session_set_shared_dir(SpiceSession *session, const gchar *dir)
  * spice_session_get_proxy_uri:
  * @session: a #SpiceSession
  *
+ * Gets the @session proxy uri.
+ *
  * Returns: (transfer none): the session proxy #SpiceURI or %NULL.
  * Since: 0.24
  **/
@@ -2594,6 +2661,7 @@ SpiceAudio *spice_audio_get(SpiceSession *session, GMainContext *context)
 /**
  * spice_usb_device_manager_get:
  * @session: #SpiceSession for which to get the #SpiceUsbDeviceManager
+ * @err: (allow-none): a return location for #GError, or %NULL.
  *
  * Gets the #SpiceUsbDeviceManager associated with the passed in #SpiceSession.
  * A new #SpiceUsbDeviceManager instance will be created the first time this
