@@ -219,6 +219,19 @@ static void update_keyboard_focus(SpiceDisplay *display, gboolean state)
     spice_gtk_session_request_auto_usbredir(d->gtk_session, state);
 }
 
+static gint get_display_id(SpiceDisplay *display)
+{
+    SpiceDisplayPrivate *d = display->priv;
+
+    /* supported monitor_id only with display channel #0 */
+    if (d->channel_id == 0 && d->monitor_id >= 0)
+        return d->monitor_id;
+
+    g_return_val_if_fail(d->monitor_id <= 0, -1);
+
+    return d->channel_id;
+}
+
 static void update_ready(SpiceDisplay *display)
 {
     SpiceDisplayPrivate *d = display->priv;
@@ -226,6 +239,16 @@ static void update_ready(SpiceDisplay *display)
 
     ready = d->mark != 0 && d->monitor_ready;
 
+    /* If the 'resize-guest' property is set, the application expects spice-gtk
+     * to manage the size and state of the displays, so update the 'enabled'
+     * state here. If 'resize-guest' is false, we can assume that the
+     * application will manage the state of the displays.
+     */
+    if (d->resize_guest_enable) {
+        spice_main_update_display_enabled(d->main, get_display_id(display),
+                                          ready, TRUE);
+    }
+
     if (d->ready == ready)
         return;
 
@@ -244,19 +267,6 @@ static void set_monitor_ready(SpiceDisplay *self, gboolean ready)
     update_ready(self);
 }
 
-static gint get_display_id(SpiceDisplay *display)
-{
-    SpiceDisplayPrivate *d = display->priv;
-
-    /* supported monitor_id only with display channel #0 */
-    if (d->channel_id == 0 && d->monitor_id >= 0)
-        return d->monitor_id;
-
-    g_return_val_if_fail(d->monitor_id <= 0, -1);
-
-    return d->channel_id;
-}
-
 static void update_monitor_area(SpiceDisplay *display)
 {
     SpiceDisplayPrivate *d = display->priv;
@@ -293,7 +303,12 @@ static void update_monitor_area(SpiceDisplay *display)
         goto whole;
     }
 
-    update_area(display, c->x, c->y, c->width, c->height);
+    /* If only one head on this monitor, update the whole area */
+    if(monitors->len == 1) {
+        update_area(display, 0, 0, c->width, c->height);
+    } else {
+        update_area(display, c->x, c->y, c->width, c->height);
+    }
     g_clear_pointer(&monitors, g_array_unref);
     return;
 
@@ -339,6 +354,7 @@ static void spice_display_set_property(GObject      *object,
         break;
     case PROP_RESIZE_GUEST:
         d->resize_guest_enable = g_value_get_boolean(value);
+        update_ready(display);
         update_size_request(display);
         break;
     case PROP_SCALING:
@@ -364,7 +380,15 @@ static void spice_display_set_property(GObject      *object,
         scaling_updated(display);
         break;
     case PROP_KEYPRESS_DELAY:
-        d->keypress_delay = g_value_get_uint(value);
+        {
+            const gchar *env = g_getenv("SPICE_KEYPRESS_DELAY");
+            guint delay = g_value_get_uint(value);
+            if (env != NULL)
+                delay = strtoul(env, NULL, 10);
+
+            SPICE_DEBUG("keypress-delay is set to %u ms", delay);
+            d->keypress_delay = delay;
+        }
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -680,6 +704,8 @@ static LRESULT CALLBACK keyboard_hook_cb(int code, WPARAM wparam, LPARAM lparam)
  * spice_display_get_grab_keys:
  * @display: the display widget
  *
+ * Finds the current grab key combination for the @display.
+ *
  * Returns: (transfer none): the current grab key combination.
  **/
 SpiceGrabSequence *spice_display_get_grab_keys(SpiceDisplay *display)
@@ -2426,7 +2452,6 @@ static void channel_new(SpiceSession *s, SpiceChannel *channel, gpointer data)
             mark(display, primary.marked);
         }
         spice_channel_connect(channel);
-        spice_main_set_display_enabled(d->main, get_display_id(display), TRUE);
         return;
     }
 
@@ -2512,12 +2537,14 @@ static void channel_destroy(SpiceSession *s, SpiceChannel *channel, gpointer dat
  * @session: a #SpiceSession
  * @channel_id: the display channel ID to associate with #SpiceDisplay
  *
+ * Creates a new #SpiceDisplay widget.
+ *
  * Returns: a new #SpiceDisplay widget.
  **/
-SpiceDisplay *spice_display_new(SpiceSession *session, int id)
+SpiceDisplay *spice_display_new(SpiceSession *session, int channel_id)
 {
     return g_object_new(SPICE_TYPE_DISPLAY, "session", session,
-                        "channel-id", id, NULL);
+                        "channel-id", channel_id, NULL);
 }
 
 /**
@@ -2526,6 +2553,8 @@ SpiceDisplay *spice_display_new(SpiceSession *session, int id)
  * @channel_id: the display channel ID to associate with #SpiceDisplay
  * @monitor_id: the monitor id within the display channel
  *
+ * Creates a new #SpiceDisplay widget associated with the monitor id.
+ *
  * Since: 0.13
  * Returns: a new #SpiceDisplay widget.
  **/
@@ -2540,7 +2569,7 @@ SpiceDisplay* spice_display_new_with_monitor(SpiceSession *session, gint channel
 
 /**
  * spice_display_mouse_ungrab:
- * @display:
+ * @display: a #SpiceDisplay
  *
  * Ungrab the mouse.
  **/
@@ -2553,7 +2582,7 @@ void spice_display_mouse_ungrab(SpiceDisplay *display)
 
 /**
  * spice_display_copy_to_guest:
- * @display:
+ * @display: a #SpiceDisplay
  *
  * Copy client-side clipboard to guest clipboard.
  *
@@ -2574,7 +2603,7 @@ void spice_display_copy_to_guest(SpiceDisplay *display)
 
 /**
  * spice_display_paste_from_guest:
- * @display:
+ * @display: a #SpiceDisplay
  *
  * Copy guest clipboard to client-side clipboard.
  *
@@ -2595,7 +2624,7 @@ void spice_display_paste_from_guest(SpiceDisplay *display)
 
 /**
  * spice_display_get_pixbuf:
- * @display:
+ * @display: a #SpiceDisplay
  *
  * Take a screenshot of the display.
  *
